#include "elementovisual.h"
#include "wx/dcbuffer.h" /* Para poder usar un buffered paint */

ElementoVisual::ElementoVisual():Elemento()
{	/*	Tipo de letra para los letreros con el nombre de los 
		elementos. */
	wxFont fuente(12, wxFONTFAMILY_ROMAN, wxITALIC, wxBOLD, false);
	
	/*	En principio todo elemento es visible */
	es_visible 		= true;
	
	estado_actual 	= INACTIVO ;
	
	/* Estas seales deben ser correctamente inicializadas en
		los constructores de los elementos o todo fallar */
	senalReposo 	= NULL;
	senalActivacion	= NULL;
	paginaPadre		= NULL;
}

ElementoVisual::~ElementoVisual()
{
	/*	El destructor no debe destruir nada. Toda la destruccion de
		elementos se lleva a cabo por la clase sistema. */
	wxLogMessage ("Destruyendo elemento "+nombreDelElementoVisual);
//	delete elemento;
}

/* 	Este mtodo implementa cosas comunes, aunque luego las clases hija
	aaden algunas cosas ms y deben recordar llamar a este metodo.
	SetElementoEnvuelto hace que el elementovisual recubra a un cierto
	elementosimple, recuperando las caracteristicas ms comunes */
void ElementoVisual::setElementoEnvuelto (Elemento* e)
{
	assert (e!=NULL);
	delete elementoEnvuelto;
	elementoEnvuelto = NULL;
	elementoEnvuelto = e;
	/* Se recupera el nombre */
	const char* nombrerecuperado = e->getNombreElemento();
	SetNombre (nombrerecuperado);
	wxLogMessage ("Nombre ha sido obtenido: %s", nombrerecuperado);
	
	/* Y la posicion */
	int x=e->getX();
	int y=e->getY();
	wxLogMessage ("Posiciones recuperadas");
	this->setX(x);
	this->setY(y);
	wxLogMessage ("Posiciones obtenidas para %s x=%i, y=%i", 
		nombrerecuperado, this->x, this->y);
}

Elemento* ElementoVisual::getElementoEnvuelto()
{
	return elementoEnvuelto;
}
void ElementoVisual::ConmutarEntrada()
{
	int codigo=-1;
	if (estado_actual == INACTIVO)
	{
		manejarEntrada (senalActivacion, POS_ENTRADA);
		codigo = elementoEnvuelto->setEntrada (senalActivacion);
		assert (codigo == 0);
		
	}
	else if (estado_actual == ACTIVO)
	{
		manejarEntrada (senalReposo, POS_ENTRADA);
		codigo = elementoEnvuelto->setEntrada (senalReposo);
		assert (codigo == 0);
	}
	assert (codigo!=-1);
	
}

void ElementoVisual::DibujarElemento (wxDC& dc)
{
//	wxMessageBox ("Intentando dibujar");
	bool seUsanTransparenciasEnBitmaps=false;
	
	/*	Metemos la imagen en las coordenadas dadas */
	
	/*	Este bitmap es distinto en funcion del elemento!*/
	dc.DrawBitmap (bitmapEnPantalla, x, y, seUsanTransparenciasEnBitmaps);
	
//	dc.SetFont (fuente);
	wxPoint pos_texto(x-4, y-12);
	dc.DrawText (nombreDelElementoVisual, pos_texto);
	if (estado_actual == SELECCIONADO)
	{
		//wxColour blanco(255,255,255);
		wxBrush	brocha(*wxCYAN, wxCROSS_HATCH );
		dc.SetBrush (brocha);
		/*	Se rellena un rea delimitada por un borde de color blanco
			con una brocha cyan que dibuja cruces diagonales (wxCROSS) */
//		dc.FloodFill (x, y, blanco, wxFLOOD_BORDER);

//		wxLogMessage ("Rellenado!");
		dc.DrawRectangle (x, y,ANCHURA_ICONOS, ALTURA_ICONOS);
	}
}
/*	Este metodo dibuja en el contexto las relaciones correspondientes */
void ElementoVisual::DibujarRelaciones (wxDC& dc)
{
	Suscripcion** suscripciones=elementoEnvuelto->getSuscripciones();
	int total_suscripciones = elementoEnvuelto->getTotalSuscripciones();
	
	for (int i =0 ; i< total_suscripciones ; i++ )
	{
		if (suscripciones[i] != NULL )
		{
			
//			wxLogMessage ( nombreDelElementoVisual + " relacionado con algo");

			Elemento* elementoReceptorSenales = 
				suscripciones[i]->getElementoSuscriptor();
			
			/*	Con esto se evita que los elementos dibujen
				las suscripciones virtuales que hace que envien sus
				salidas hacia el sistema contenedor */
			if (elementoReceptorSenales!=contenedorPadre)
			{
				DibujarSuscripcion ( elementoReceptorSenales, dc );
			}
		}
	}
}

void ElementoVisual::Seleccionar()
{
	wxLogMessage ("Seleccionado el elemento:"+nombreDelElementoVisual);
	estado_actual = SELECCIONADO;
}
void ElementoVisual::Deseleccionar()
{
	wxLogMessage ("Deseleccionado el elemento:"+nombreDelElementoVisual);
	estado_actual = INACTIVO;
}

void ElementoVisual::Configurar(Elemento* elemento)
{
	/*	Se obtiene un dialogo especfico */
	dialogoDeConfiguracion = this->FabricarDialogo();
	
	/*********************INTENTO***********************/
	//EL null indica que no hay padre, y el -1 que nos da igual el ID del dialogo
	dialogoDeConfiguracion->Create (NULL, -1);
	/****************************************************/

	if (dialogoDeConfiguracion==NULL) wxMessageBox ("Dialogo es NULL");
	/*	Se cargan en el dialogoDeConfiguracion las propiedades del elemento */
	dialogoDeConfiguracion->getPropiedades (elemento);

	/*	Si el usuario acaba por pulsar OK...*/
	if (dialogoDeConfiguracion->ShowModal() == wxID_OK )
	{
		/*	Se cargan los valores del dilogo en el elemento */
		dialogoDeConfiguracion->setPropiedades (elemento);
		
		/* Y en el elemento visual se actualiza el nombre inmediatamente
			ya que es algo que se muestra en pantalla */
		SetNombre (elemento->getNombreElemento());
		
	}	
	
	/*	Y se destruye el dialogoDeConfiguracion */
	dialogoDeConfiguracion->Destroy();
	delete dialogoDeConfiguracion;
}

void ElementoVisual::Configurar()
{
//	wxMessageBox ("Configurando el elemento");
	Configurar (elementoEnvuelto);
	return ;
}

int	ElementoVisual::ElegirSalidaDeEmisor()
{	/* Esta es la salida por defecto */
	return POS_SALIDA;
}

int ElementoVisual::ElegirEntradaDeReceptor()
{
	return POS_ENTRADA;
}

wxPoint	ElementoVisual::GetXY()
{
	return wxPoint (x, y);
}
void ElementoVisual::SetXY(wxPoint punto)
{
	x=punto.x;
	y=punto.y;
	setX(x);
	setY(y);
}
void ElementoVisual::SetXY(int x, int y)
{
	this->x=x;
	this->y=y;
	setX(x);
	setY(y);
}
void ElementoVisual::SetVisible (bool visible)
{
	es_visible = visible;
}

bool ElementoVisual::EsVisible ()
{
	return es_visible ;
}


wxRect ElementoVisual::GetRectangulo()
{
	wxPoint esquina_sup_izq (x,y);
	wxPoint esquina_inf_der (x+ANCHURA_ICONOS, y+ ALTURA_ICONOS);
	
	return wxRect (esquina_sup_izq, esquina_inf_der);
}
void ElementoVisual::SetNombre (const char *nombre)
{
	/*	No es una asignacin. Internamente hay una construccin de copia */
	nombreDelElementoVisual = nombre;
	this->setNombreElemento ( nombre );
	string nombreEscrito (getNombreElemento());
//	wxLogMessage ("Nombre escrito:%s", nombreEscrito.c_str());
}
void ElementoVisual::SetNombre (wxString nombre)
{
	/*	No es una asignacin. Internamente hay una construccin de copia */
	nombreDelElementoVisual = nombre;
	setNombreElemento(nombre.c_str());
	this->setNombreElemento ( nombreDelElementoVisual.c_str() );
}

wxString ElementoVisual::GetNombre()
{
	return nombreDelElementoVisual;
}

void ElementoVisual::SetSufijo (int numelemento)
{
	wxLogMessage ("----ElementoVisual::SetSufijo----");
	wxLogMessage ("Parametro (numelemento %i)", numelemento);
	wxLogMessage ("Fichero %s, linea %i", __FILE__, __LINE__);
	wxString nombre(nombreDelElementoVisual);
	nombre = nombre + wxString::Format("%d", numelemento);
	SetNombre (nombre);
	wxLogMessage ("Estableciendo nombre del elemento visual...");
	this->setNombreElemento (nombre.c_str());
	wxLogMessage ("Estableciendo nombre del elemento envuelto...");
	assert (elementoEnvuelto!=NULL);
	elementoEnvuelto->setNombreElemento (nombre.c_str());
	wxLogMessage ("Establecido!");
//	wxLogMessage ("Se pone el nombre "+nombre);
}
Elemento* ElementoVisual::GetPadre ()
{
	return this->getSistemaPadre();
}

void ElementoVisual::SetPadre (Elemento* padre)
{
	assert (padre!=NULL);
	this->setSistemaPadre (padre);
}


void ElementoVisual::DibujarSuscripcion (Elemento* e,wxDC& contexto)
{
	if (e==NULL)
	{
		wxMessageBox ("Error de programa: Suscripcin inexistente (es NULL)");
		return ;
	}
	if (!e->esVisible())
	{
//		wxLogMessage("No se puede pintar suscripcion hacia un invisible");
		return ;
	}
	//wxPoint inicial( x + (ANCHURA_ICONOS/2), y + (ANCHURA_ICONOS/2) );
	wxPoint inicial;
	wxPoint final;

	wxString nombreHacia (e->getNombreElemento());
//	wxLogMessage ("Suscripcion de "+nombreDelElementoVisual+" hacia "+ nombreHacia);
	inicial = 	wxPoint(x, y);
	final	=	wxPoint (e->getX(), e->getY());;
	
	#define LONGITUD_LINEA_FINAL	25
	/*	La linea est hecha de tres segmentos, uno horizontal
		hasta la mitad de la distancia, una vertical hasta la 
		altura del segundo elemento y una tercera horizontal
		que conecta con el elemento */
	int x_inicial		= inicial.x + (ANCHURA_ICONOS/2);
	int y_inicial		= inicial.y + (ALTURA_ICONOS/2);
	int x_intermedio 	= (final.x + inicial.x) / 2;
	int y_intermedio 	= (final.y + inicial.y) / 2;
	int x_final			= final.x;
	int y_final			= final.y + (ALTURA_ICONOS/2);
	contexto.DrawLine (x_inicial, y_inicial, x_inicial+40, y_inicial);
	contexto.DrawLine (	x_inicial+40, y_inicial, x_intermedio, y_intermedio);
	contexto.DrawLine ( x_intermedio, y_intermedio, 
			x_final-LONGITUD_LINEA_FINAL, y_final);
	contexto.DrawLine ( x_final-LONGITUD_LINEA_FINAL, y_final, 
		x_final, y_final);
	
	/*Con esto se pinta una punta de flecha*/
	#define ANCHURA_FLECHA	10 	//Anchura en pixeles de la flecha
	#define ALTURA_FLECHA	5	//Altura en pixeles
	wxPoint puntos_triangulo[4];
	puntos_triangulo[0] = wxPoint (x_final, y_final);
	puntos_triangulo[1] = wxPoint (x_final-ANCHURA_FLECHA, y_final-ALTURA_FLECHA );
	puntos_triangulo[2] = wxPoint (x_final-ANCHURA_FLECHA, y_final+ALTURA_FLECHA );
	puntos_triangulo[3] = wxPoint (x_final, y_final);
	wxBrush color_relleno_flecha(*wxBLACK);
	contexto.SetBrush (color_relleno_flecha);
	contexto.DrawPolygon (4, puntos_triangulo);
}


int ElementoVisual::Conectar
	(Elemento* e,int pos_entrada,	int pos_salida)
{
	assert (e!=NULL);
	assert (pos_entrada>=0);
	assert (pos_salida>=0);
	assert (elementoEnvuelto!=NULL);
	wxLogMessage ("----ElementoVisual::Conectar----");
	wxLogMessage ("Parametros: e no es null (nombre %s), pos_entrada=%i, pos_salida=%i",
		e->getNombreElemento(),pos_entrada, pos_salida);
	/*	Si el tipo de la seal de salida no es el mismo que el tipo que el
		suscriptor tiene a la entrada no se permite unir esos elementos */
	wxLogMessage ("Listos para recuperar el tipo de la salida %i", pos_salida);
	int tipoSalida = elementoEnvuelto->getTipoSalida(pos_salida);
	wxLogMessage ("Tipo recuperado: %i", tipoSalida);

	int tipoEntrada= e->getTipoEntrada(pos_entrada);
	wxLogMessage ("Salida(%i),Tipo de salida %i, Entrada(%i),tipo de entrada %i", 
		pos_salida, tipoSalida, pos_entrada, tipoEntrada);

	/* A veces el tipo de senal a conectar no coincide, p.ej porque se
		intenta conectar un pulsador a un fotosensor. En ese caso se obtiene
		un mensaje de error. Sin embargo, si no coinciden y uno de los tipos
		de seal es NADA, es porque se intenta conectar un elemento y
		un conector, lo que s est permitido */
	if (tipoEntrada!=tipoSalida)
	{
		if ( (tipoEntrada==NADA) || (tipoSalida!=NADA) )
		{
			/* Es ms legible dejar este bloque vaco que poner
				la condicin contraria */
			wxLogMessage ("Conectando elemento");
		}
		else{
			wxLogMessage ("Las seales de salida y entrada no son del mismo tipo");
		}
	}

	/*	Se llama al metodo de la implementacion */
	int codigo_error =elementoEnvuelto->anadirSuscriptor (e, pos_entrada, pos_salida);
//		elemento->anadirSuscriptor (e, pos_entrada, pos_salida);
		
	switch (codigo_error)
	{
		case POS_ENTRADA_INVALIDA:
			{
				wxLogMessage ("No se puede conectar a esa entrada");
				break;
			} 
		case POS_SALIDA_INVALIDA:
			{
				wxLogMessage ("No se puede conectar a esa salida");
				break;
			} 
		case ELEMENTO_INVALIDO:
			{
				wxLogMessage ("El elemento receptor es NULL");
				break;
			} 
		case DEMASIADAS_SUSCRIPCIONES:
			{
				wxLogMessage ("El elemento que ha seleccionado ha\
					superado el limite de elementos a los que puede avisar:");
				wxLogMessage ("Modifique el parmetro MAX_SUSCRIPCIONES \
					en el fichero constantes.h y recompile todo");								
				break;
			} 
		case 0:
			{
				wxString nombre(e->getNombreElemento());
				wxLogMessage ("Interconexion realizada con "+nombre);
				break;
			} 
		default:
			{
				wxLogMessage ("anadirSuscriptor de la libreria elementos.a\
					ha devuelto un codigo de error desconocido(%i)", codigo_error);
			}
	}
	assert (codigo_error==0);
	return codigo_error;
}

int ElementoVisual::Desconectar
	(Elemento* e,int pos_entrada,	int pos_salida)
{
//	elemento->eliminarSuscriptor (e, pos_entrada, pos_salida);
	int codigo = elementoEnvuelto->eliminarSuscriptor (e, pos_entrada, pos_salida);
	
	return codigo;
}

wxString ElementoVisual::getInformacion()
{
	wxString cadenaADevolver;
	string informacion=string (elementoEnvuelto->getInformacionElemento());
	
	cadenaADevolver=wxString (informacion.c_str());
	return cadenaADevolver;
}

NodoXML ElementoVisual::getXML ( DOMImplementation* impl )
{
	wxLogMessage ("---ElementoVisual::getXML()---");
	wxLogMessage ("Recuperando XML de %s (totalSuscripciones %i)", 
		nombreElemento, totalSuscripciones);
	return elementoEnvuelto->getXML(impl);
}

void ElementoVisual::getXMLSuscriptores (DOMDocument *documento_xml)
{
	wxLogMessage ("---ElementoVisual::getXmlSuscriptores---");
	wxLogMessage ("Para el elemento %s", nombreElemento);
	elementoEnvuelto->getXMLSuscriptores (documento_xml);
}

void 	ElementoVisual::SetVentanaPadre (wxWindow* padre)
{
	paginaPadre = padre;
}

void ElementoVisual::asignarNombreUnico (const char* nombreDeElemento, int numero)
{
	/*	La implementacion no es muy sofisticada pero no se sabia si el sistema
		de nombrado se iba a modificar, por eso se separa en un mtodo aparte */
		
	/*	Tener en cuenta que hay un espacio detrs */
	wxString cadenaNumero = wxString::Format (" %i", numero );
	wxString cadenaNombre = wxString (nombreDeElemento) + cadenaNumero;
	
	nombreDelElementoVisual = cadenaNombre;
}
