#ifndef _ventana_h
#define _ventana_h

#include "panel_elementos.h"
#include "parser.h"
#include "sistema.h"
#include "eventoclicknotebook.h"
#include "constantes.h"
#include "dialogos/dialogo.h"
#include "elementosvisuales/elementovisual.h"
#include "elementosvisuales/vectorvisual.h"
#include "pagina.h"
#include "dialogos/dialogoxml.h"

#include "wx/toolbar.h"
#include "wx/file.h"
class Ventana : public wxFrame
{
	public:
		Ventana();
		~Ventana();
	private:
		/*	Esta variable indica si estamos en modo construccion (true)
			o en modo simulacion (false) */
		bool modoSimulacion;
		
		/*	Indica si el usuario ha pulsado algun boton en la barra de
			herramientas */
		bool barraDeHerramientasEstaActiva;
		
		wxMenuBar* crearMenus();
		wxBoxSizer* contenedorPrincipal;
		wxBoxSizer* contenedorArriba;
		wxBoxSizer* contenedorAbajo;
		
		/*	En esta posicin es donde por defecto se insertan
			todos los elementos */
		wxPoint			pos_insercion_por_defecto;
		
		wxMenuBar*	barraMenu;
		wxMenu*		menuArchivo;
		wxMenu*		menuEdicion;
		wxMenu*		menuVer;
		wxMenu*		menuModo;
		wxMenu*		menuInsertar;
		wxMenu*		menuAyuda;
		
		/*	Nombre del archivo que maneja la ventana*/
		wxString	nombreArchivo;
		int		elementoActivo;
		
		/*	Nos dice cuantos subsistemas llevamos creados. Solo se
			utiliza para dar nombres a los subsistemas */
		int subsistemas;
		/*	Este cuadro de texto mostrar mensajes. Se muestra en
			la parte inferior de la aplicacion*/
		wxTextCtrl*	texto;
		
		/*	Este control gestiona las pestaas de los diferentes
			subsistemas */
		wxNotebook*	libro;
		
		/* Contiene todos los botones de los distintos elementos */
		PanelElementos* botonera;
		
		/*	Elementos que hay para crear un vector de controles */
		wxCheckBox *chkVector;
    	wxComboBox *cmbPosicion;
    	wxStaticText *lblSeparacion;
    	wxStaticText* lblNumElementos;
    	wxSpinCtrl *spnSeparacion;
    	wxSpinCtrl *spnNumElementos;
				
		/*	Contiene el numero de pagina con el que se trabaja */
		int			paginaActual;
		
		/*	Contiene el numero total de pginas que hay en el libro */
		int			totalPaginas;
		
		/*	Este es el sistema con el que se est trabajando
			en un cierto momento */
		Sistema*	sistemaActivo;
		
		/*	Aqui se almacenan los distintos tipos de dilogo
			de los distintos tipos de elemento que sern invocados
			cuando se aada un elemento */
		Dialogo* 	dialogoDeConfiguracion[TOTAL_ELEMENTOS];
		
		/* 	Estos son los identificadores de eventos
			de pulsacin en los menus */
		enum
		{
			MENU_ARCHIVO_NUEVO,
			MENU_ARCHIVO_ABRIR,
			MENU_ARCHIVO_GUARDAR,
			MENU_ARCHIVO_GUARDAR_COMO,
			MENU_ARCHIVO_SALIR,
			MENU_EDICION_CORTAR,
			MENU_EDICION_COPIAR, 
			MENU_EDICION_PEGAR,
			MENU_EDICION_ELIMINAR_ELEMENTO,
			MENU_EDICION_ELIMINAR_SISTEMA,
			MENU_VER_XML, 
			MENU_MODO_CONSTRUCCION,
			MENU_MODO_SIMULACION,
			MENU_INSERTAR_SISTEMA,
			MENU_INSERTAR_SISTEMA_DE_PRUEBA,
			MENU_INSERTAR_SISTEMA_DE_PRUEBA2,
			MENU_INSERTAR_SISTEMA_DE_PRUEBA3,
			MENU_AYUDA_ACERCA_DE,
			//	Esto es para la creacion de vectores */
			ID_CREAR_VECTOR, ID_COMBO_POSICION, ID_SEPARACION_VECTORES,
			ID_NUMELEMENTOS
		};
		
		/*	Este metodo activa o desactiva los elementos que
			permiten crear vectores de elementos visuales */
		void	ConmutarVectorizacion();
	
		/*	Cuando alguien pulsa el boton Sistema, se deben hacer
			unas cuantas cosas. Este mtodo crea pginas nuevas 
			en el notebook y les aade el subsistema creado */
		void 		CrearPaginaSubsistema (SistemaVisual *s);
		
		
		/*	Y esta variable es la que indica si algo est habilitado o no */
		bool	vectoresHabilitados;
		/*	Este metodo es usado por los mens de guardar */
		void 		Guardar					( wxString& nombre );
		/*	Nombre del fichero con el que se est trabajando */
		wxString	ObtenerNombreDeFichero	( );	
		
		/* 	Funciones gestoras de eventos */
		void OnMenuArchivoAbrir 		( wxCommandEvent& evento );
		void OnMenuArchivoGuardar 		( wxCommandEvent& evento );
		void OnMenuArchivoGuardarComo 	( wxCommandEvent& evento );
		void OnMenuEdicionEliminarElemento( wxCommandEvent& evento );
		void OnMenuEdicionEliminarSistema( wxCommandEvent& evento );
		void OnMenuInsertarSistema 		( wxCommandEvent& evento );
		void OnMenuVerXML				( wxCommandEvent& evento );
		void OnMenuAyudaAcercaDe		( wxCommandEvent& evento );
		/*	Esto permite insertar un sistema de prueba en una pagina */
		void OnMenuInsertarSistemaDePrueba(wxCommandEvent& evento);
		void OnMenuInsertarSistemaDePrueba2(wxCommandEvent& evento);
		void OnMenuInsertarSistemaDePrueba3(wxCommandEvent& evento);
		void OnMenuModoConstruccion		( wxCommandEvent& evento );
		void OnMenuModoSimulacion		( wxCommandEvent& evento );
		
		/* 	Estos gestores de eventos corresponden a la pulsacion
			de botones de la barra de herramientas */
		void OnPulsadorActivo			( wxCommandEvent& 		evento 	);
		void OnReleActivo				( wxCommandEvent& 		evento 	);
		void OnBiestableActivo			( wxCommandEvent& 		evento 	);
		void OnReedActivo				( wxCommandEvent& 		evento 	);
		void OnContadorActivo			( wxCommandEvent& 		evento 	);
		void OnZumbadorActivo			( wxCommandEvent& 		evento 	);
		void OnTemporizadorActivo		( wxCommandEvent& 		evento 	);
		void OnElectroimanActivo		( wxCommandEvent& 		evento 	);
		void OnLamparaActivo			( wxCommandEvent& 		evento 	);
		void OnMotorActivo				( wxCommandEvent& 		evento 	);
		void OnSistemaActivo			( wxCommandEvent& 		evento 	);
		void OnFotosensorActivo			( wxCommandEvent& 		evento	);
//		void OnPaint 					( wxPaintEvent& 		evento	);
		void OnClickEnNotebook 			( wxEvtClickNotebook& 	evento	);
		void OnPagina					( wxNotebookEvent&		evento	);
		void OnClickCrearVector			( wxCommandEvent& 		evento	);
		/* 	Aqui se incluye la declaracin de la tabla
			de eventos */
		DECLARE_EVENT_TABLE()
		
		/*	Estas son funciones de uso privado y de ayuda. Se han puesto en
			ficheros aparte */
			
		/*	Inserta el elemento en el panel de la pestaa correspondiente
			e inserta el bitmap que lo representa*/
		ElementoVisual* insertarObjetoEnSistema ( 	
						int tipoDeElemento, wxWindow* ventana,
						int x, int y );
		/*	Deshace todas las selecciones y conexiones pendientes que hubiera
			en la pagina en la que se trabaja .Nota, quiza tambin se pueda
			hacer en el resto de pginas.*/
		void ReiniciarEstadoPaginaActual();										 
		/*	Solo se incluyen estos metodos si estamos depurando elementos*/
		void Localizar(wxMouseEvent& evento);
		/* Metodos de prueba */
		void TestTemporizador	();
		void TestElectroiman	();
		void TestPulsador		();
		void TestFotoSensor		();
		void TestMotor			();
		void TestContador		();
		void TestReed			();
		void TestBiestable		();
		void TestZumbador		();
		void TestRele			();
		void TestLampara		();
		
		/*	Esta funcion conmuta el estado de las paginas para que
			se pongan a simular o parar los sistemas que contienen.
			En teoria es posible seguir diseando un sistema mientras
			el de otra pgina sigue funcionando pero tal situacion no
			se ha probado */
		void ConmutarSimulacion (bool modoSimulacion);
		
		/*	Este metodo recupera el sistema que hay dentro
			de un fichero XML */
		Sistema* 		RecuperarSistemaDesdeXML(Parser* p, wxWindow* padre);
		Sistema* 		ProcesarXML(DOMNode* elementoXML, wxWindow* padre);
		ElementoVisual*	ObtenerElementoVisual(
				char* nombreetiquetaXML, wxWindow* pagina,
				DOMElement* elemento, Sistema* sistemaDevuelto);
		void			CrearRelacion(DOMElement* elemento_relacion, Sistema* s);
};
#endif
