#include "electroiman.h"
#include "constantes.h"

#include <iostream>
using namespace std;


Electroiman::Electroiman ()
{
	voltajeEntradaReposo	=	NULL;
	voltajeEntradaActivacion=	NULL;
	magSalidaReposo			=	NULL;
	magSalidaActivacion		=	NULL;
	
	entradas[POS_ENTRADA]= NULL;
	salidas [POS_SALIDA ]= NULL;
}
Electroiman::Electroiman(Voltaje* vEntradaReposo, Voltaje* vEntradaActivacion,
			Magnetismo* mSalidaReposo, Magnetismo* mSalidaActivacion)
{
	voltajeEntradaReposo	=	vEntradaReposo;
	voltajeEntradaActivacion=	vEntradaActivacion;
	magSalidaReposo			=	mSalidaReposo;
	magSalidaActivacion		=	mSalidaActivacion;
	
	entradas[POS_ENTRADA]= vEntradaReposo;
	salidas [POS_SALIDA ]= mSalidaReposo;
}

Voltaje* Electroiman::getVoltajeEntradaReposo()
{
	return voltajeEntradaReposo;
}

Voltaje* Electroiman::getVoltajeEntradaActivacion()
{
	return voltajeEntradaActivacion;
}

Magnetismo* Electroiman::getMagSalidaReposo()
{
	return magSalidaReposo;
}

Magnetismo* Electroiman::getMagSalidaActivacion()
{
	return magSalidaActivacion;
}


void	Electroiman::setVoltajeEntradaReposo		( Voltaje*	voltaje)
{
	voltajeEntradaReposo=voltaje;
}
void	Electroiman::setVoltajeEntradaActivacion	( Voltaje*	voltaje)
{
	voltajeEntradaActivacion=voltaje;
}
void 	Electroiman::setMagSalidaReposo			( Magnetismo*	magnetismo)
{
	magSalidaReposo=magnetismo;
}
void	Electroiman::setMagSalidaActivacion		( Magnetismo*	magnetismo)
{
	magSalidaActivacion=magnetismo;
}
		

int Electroiman::setEntrada (Senal* entrada, int posicion)
{
//	assert (false);
	assert (entrada!=NULL);
	assert (posicion<=totalEntradas);
	assert (posicion>=0);
	
	Voltaje *voltEntradaActual=(Voltaje*) entrada;
	
	if (voltEntradaActual->getValor() == voltajeEntradaReposo->getValor() )
	{
		entradas[POS_ENTRADA]	=	voltajeEntradaReposo;
		salidas [POS_SALIDA] 	=	magSalidaReposo;
		return (Elemento::setEntrada (entrada,posicion) );
	}
	
	if (voltEntradaActual->getValor() == voltajeEntradaActivacion->getValor() )
	{
		entradas[POS_ENTRADA]	=	voltajeEntradaActivacion;
		salidas [POS_SALIDA] 	=	magSalidaActivacion;
		return (Elemento::setEntrada (entrada,posicion) );
	}
	 
	 /*	Cuidado!. Si se llega a este punto es porque la entrada, no 
	 	era vlida, por lo que simplemente se toma nota de la entrada
	 	pero no se produce ninguna variacion en la salida. El problema
	 	habr ocurrido en el llamador de esta funcion, que no ha 
	 	suministrado un valor correcto*/
	 	
	entradas[POS_ENTRADA]=voltEntradaActual;
	return (Elemento::setEntrada (entrada,posicion) );
}

NodoXML Electroiman::getXML ( DOMImplementation *impl )
{
	//etqElemento se hereda por parte de XMLSerializable
	//etqElemento = XMLString::transcode ( "electroiman" );
	
	DOMDocument* doc = impl->createDocument ();
	cout << "Creando documento base XML para electroimans\n";
	doc = getDocumentoXMLBase ( doc, "electroiman", nombreElemento, x, y );
	
	/*	Ahora aadimos a doc los elementos XML para indicar las distintas
		caractersticas. No se debe alterar el orden de creacin de elementos
		ya que el esquema exige dicho orden */
	cout << "Documento base electroimans creado\n";
	anadirElementoXML (doc, "entradareposo", 		
								getVoltajeEntradaReposo() 			);
	anadirElementoXML (doc, "salidareposo", 		
								getMagSalidaReposo()			);
	cout << "XML electroimans:Reposo creados\n";
	anadirElementoXML (doc, "entradaactivacion",	
								getVoltajeEntradaActivacion() 		);
	anadirElementoXML (doc, "salidaactivacion",		
								getMagSalidaActivacion() 	);
	cout << "XML electroimans:Activacion creada\n";
	
	DOMDocumentFragment* fragmento = doc->createDocumentFragment ();
	fragmento->appendChild ( doc->getDocumentElement() );
	return fragmento;
}

void Electroiman::construir (DOMElement* nodo)
{
	obtenerDatosComunes (nodo);
	
	/* Obtenemos los valores de los distintos nodos */
	voltajeEntradaReposo	= (Voltaje*) getSenal 
					( VOLTAJE, "entradareposo", 		nodo );
	magSalidaReposo			= (Magnetismo*) getSenal
					( MAGNETISMO, "salidareposo",		nodo );
	voltajeEntradaActivacion= (Voltaje*) getSenal 
					( VOLTAJE, "entradaactivacion", 	nodo );
	magSalidaActivacion		= (Magnetismo*) getSenal
					( MAGNETISMO, "salidaactivacion",	nodo );
	
	entradas[POS_ENTRADA]	=voltajeEntradaReposo;
	salidas[POS_SALIDA]		=magSalidaReposo;
}

