#ifndef _elemento_
#define _elemento_

#include "senal.h"
#include "constantes.h"
#include "xmlserializable.h"
#include "creadorvalores.h"
#include "codificador.h"
#include "suscripcion.h"
#include <map>
#include <string>
#include <vector>
#include "correspondencia.h"

/*	Definida mas abajo*/
class Suscripcion; 

using namespace std; /* Para poder escribir string en vez de std::string*/

/*	Para hacer los elementos lo mas genericos posibles se han utilizado vectores
	de entradas y salidas, reservndose las primeras posiciones para las
	entradas ENABLE, RESET y KEEP. Observar el fichero de constantes.h para
	mas informacion*/
class Elemento: public XMLSerializable {
	public:
	/*	Constructor: Se puede ver lo que hace en elemento.cpp*/
	Elemento();
	virtual ~Elemento();
	void 		setSistemaPadre (Elemento* padre);
	Elemento*	getSistemaPadre();
	/*	Si el llamador no pone nada se modifica por defecto POS_ENTRADA,
		Se hace as porque la mayor parte de los elementos solo 
		tiene una entrada*/
	virtual int setEntrada (Senal* entrada, int posicion=POS_ENTRADA);
	
	/*	Este metodo PUEDE ser redefinido en los hijos para gestionar
		las entradas por ejemplo, para poder depurar. Es el paso especfico que
		se tiene/puede hacer diferente en cada objeto */
	virtual void manejarEntrada (Senal* entrada, int posicion);
	
	/*	Cuando otro elemento se convierte en suscriptor de este elemento
		esta conectando una cierta entrada suya a una cierta salida nuestra.
		Tambien se usan parametros por defecto*/
	virtual int anadirSuscriptor   (Elemento* e,int pos_entrada=POS_ENTRADA, 
							int pos_salida=POS_SALIDA);
	virtual int eliminarSuscriptor (Elemento* e,int pos_entrada=POS_ENTRADA, 
							int pos_salida=POS_SALIDA);
	
	/*	Esto borra todas las suscripciones que pueda haber
		y que tengan como avisador a un elemento y como suscriptor
		a un elemento e*/
	virtual int eliminarSuscripcionesAElemento (Elemento *e);
	/*	Manipulacion del nombre del elemento */
	void setNombreElemento (const char* nombre);
	/* 	Es importante que se devuelva un const char* para evitar que alguien
		modifique el nombre "desde fuera" usando el puntero */
	const char* getNombreElemento();
	
	virtual int 			getTotalEntradas();
	virtual int 			getTotalSalidas ();
	virtual int 			getTotalSuscripciones();
	virtual Suscripcion**	getSuscripciones();
	
	/* Nos dicen si una entrada o salida es un voltaje o giro de motor...*/
	virtual int 	getTipoEntrada(int posicion=POS_ENTRADA);
	virtual int 	getTipoSalida (int posicion=POS_SALIDA);
	
	/* Para leer las seales a la entrada y salida de un elemento*/
	Senal* 			getEntradaPorPosicion(int posicion=POS_ENTRADA);
	Senal* 			getSalidaPorPosicion (int posicion=POS_SALIDA);
	
	/*	Las primeras entradas estan reservadas para las
		entradas DEBUG, RESET y similares. Estas funciones
		nos devuelven la primera entrada "real"*/
	virtual int getPrimeraEntradaUtil();
	virtual int getPrimeraSalidaUtil();
	virtual int getPrimeraEntradaUtil(const char* nombre);
	virtual int getPrimeraSalidaUtil (const char* nombre);
	/*	Estos metodos van sobrecargados en las clases hija. Permiten
		buscar la primera entrada de un elemento dado su nombre*/
	virtual int getPrimeraEntrada(const char* nombre);
	virtual int getPrimeraSalida (const char* nombre);
	

	void pintarSuscripciones();
	
	string getInformacionElemento();
	
	/*	Estas funciones devuelven las coordenadas del elemento en el plano */
	int getX();
	int getY();
	
	/*	Estas las establecen */
	
	void setX (int x);
	void setY (int x);	
	
	/*	Un Elemento solo es la base para los elementos concretos, as
		que estos mtodos tienen una implementacion vaca */
	virtual NodoXML getXML ( DOMImplementation* impl ) { return NULL;} ;
	
	/*	Este mtodo aade la descripcion en XML de los suscriptores
		que reciben cambios en la entrada del elemento*/
	virtual void 	getXMLSuscriptores (DOMDocument *documento_xml);
	virtual void construir (DOMElement*){ };
	
	/*	Permite conocer el nombre de las entradas y salidas, as
		como las posiciones que ocupan */
	virtual map<string,int> getNombreEntradas	();
	virtual map<string,int> getNombreSalidas	();
	
	/*	Este mtodo actualiza el mapa de entradas o salidas */
	map<string, int>actualizarMapaNombres 		(map<string, int> &mapa);
	
	/*	Cuando se hace una suscripcin, se toma nota del elemento
		que nos va a notificar su cambio de estado. Esto se hace
		as, porque cuando destruyamos este elemento es muy fcil
		avisar a todos los elementos de que tienen que eliminar
		las suscripciones que apuntan a este elemento. Si no se
		hiciera as, la eliminacin de elementos se hace demasiado
		complicada para resultar fcil de entender. */
	void anotarAvisador		(Elemento* e);
	void eliminarAvisador 	(Elemento* e);
	
	/*	Este metodo notifica a todos los elementos de los cuales
		somos suscriptores de que se va a eliminar el elemento
		y que por tanto deben eliminar las suscripciones que
		nos apunten */
	virtual void notificarEliminacion();
	
	/*	Si el nombre del elemento coincide, nos devuelve
		una referencia al elemento */
	virtual Elemento* getRefElementoPorNombre (const char* nombreElemento);
	
	/*	Nos devuelve la entrada de un elemento dado su nombre*/
	virtual Senal* getEntradaPorNombreYPosicion (const char* nombre, int pos);
	
	/*	Nos devuelve la salida de un elemento dado su nombre*/
	virtual Senal* getSalidaPorNombreYPosicion (const char* nombre, int pos);
	
	virtual int getEntradaDeSistema (const char* nombrePasado, 	int posicion);		
	virtual int getSalidaDeSistema  (const char* nombrePasado, 	int posicion);
	
	/*	Estos dos mtodos nos dicen si a ese elemento le han indicando
		la correspondencia entre sus entradas y las entradas de un sistema
		que lo contiene*/
	virtual bool entradasEstanMapeadas();
	virtual bool salidasEstanMapeadas();
	
	/*  Con estos mtodos se le indica al mtodo cual es la primera posicion
		de sus entradas o salidas */
	virtual void setMapeoPrimeraEntrada(int pos_sistema);
	virtual void setMapeoPrimeraSalida(int pos_sistema);
	
	/*	Nos dice si un elemento es visible o no. Casi todos los son
		excepto los conectores */
	virtual bool 	esVisible();
	protected:
		Elemento* elementoPadre;
		/*	Aqui se almacena el nombre de las entradas y salidas y la 
			correspondiente posicin que ocupan en un elemento. Esto
			permite referirse por ejemplo a "EntradaR" en un biestable
			en lugar de tener que buscar o calcular la posicin
			de la misma */
		map<string, int> nombreEntradas;
		map<string, int> nombreSalidas;
		
		char nombreElemento[MAX_LONG_NOMBRE_ELEMENTO];
		Senal* entradas[MAX_LONG_VECTOR_ENTRADAS];
		Senal* salidas [MAX_LONG_VECTOR_SALIDAS];
		Suscripcion* suscripciones[MAX_SUSCRIPCIONES];
		/*	Cada elemento tendra que indicar en su constructor cuantas
			entradas y salidas tiene*/
		int totalEntradas;
		int totalSalidas;
		int totalSuscripciones;
		
		
		/*	Esta es la primera posicin que ocupa la primera
			entrada dentro de un elemento. Si por ejemplo tiene
			un valor de 5, significa que la entrada 0 de este
			elemento est en la posicin 5*/
		int pos_de_la_primera_entrada_en_sistema;
		int pos_de_la_primera_salida_en_sistema;
		/* 	Esta funcion nos dice si dos float son razonablemente
			iguales. Es necesaria porque en C 5.0 y 5.0000 pueden
			ser ligeramente diferentes en su mantisa y su comparacion
			con == podria dar FALSE*/
		bool iguales(float x, float y);
		
		/*	Nos dice si dos seales son iguales */
		bool iguales(Senal* s1, Senal* s2);
		/*	Esta es la funcin que notifica de cambios en la entrada
			a los elementos suscritos (patrn Observer) */
		void notificar();
		
		/*	Esta funcion borra una suscripcion reordenando el vector para
			que no haya elementos NULL entre las posiciones del vector*/
		void borrarSuscripcion (int posicion);
		
		/*	Este metodo se utiliza para poner en el nodo
			XML el nombre del elemento y la posicion. Como todos los
			elementos necesitan nombre y posicion, se usa para
			factorizar el comportamiento comun */
		void obtenerDatosComunes(DOMElement* nodo);
		
		/*	Estas son las coordenadas que un elemento ocupa en el plano */
		int x;
		int y;
		
		vector<Elemento*> avisadores;
		
		/*	Aqui se almacena la correspondencia entre las posiciones
			de entrada "virtuales" y las posiciones de entrada
			reales. Esto se hace asi porque hay elementos, como los
			sistemas, que pueden albergar muchos elementos en su
			interior y las correspondencia entre entradas o
			salidas no tiene por qu ser la 3 con la 3 o la 4
			con la 4*/
		vector<Correspondencia> correspondencia_de_entradas;
		vector<Correspondencia> correspondencia_de_salidas;
		void inicializarCorrespondencias();
		
		
		/*	Este mtodo es un poco mas "ciego". Devuelve la
			correspondencia que contiene informacin sobre
			una posicion de sistema*/
		//Correspondencia buscarCorrespondencia (
//			vector<Correspondencia> &v, int posicionDeSistema );
};
/*	Fin de la clase Elemento*/





#endif
