#include <xercesc/framework/LocalFileFormatTarget.hpp>

#include "parser.h"
#include "sistema.h"

#include "motor.h"
#include "lampara.h"
#include "reed.h"
#include "fotosensor.h"
#include "pulsador.h"
#include "electroiman.h"
#include "zumbador.h"
#include "contador.h"
#include "rele.h"
#include "temporizador.h"
#include "domgestionerror.h"
#include "elemento.h"
#include "biestable.h"


using namespace std;
/*Se construye un parser a partir de un fichero dado*/
Parser::Parser () 
{
	/*Se intenta inicializar el subsistema XML*/
	try {
		XMLPlatformUtils::Initialize();
	}
	catch (const XMLException& tocatch) {
		cout << "Error!" <<endl ;
	}
	
	//Observese que en gLS hay la cadena "LS\0" (null al final)
	//La cadena se tiene que fabricar asi porque Xerces usa 
	//caracteres de ancho largo (XMLCh's)
	
	//La cadena LS significa Load and Save, y permite obtener un 
	//DOMParser que pueda leer y salvar XML's
	
	
	//En Xerces 2.4 esta forma de obtener una DOMImplementation
	//es experimental y podria cambiar en otras versiones
	static const XMLCh gLS[] = { chLatin_L, chLatin_S, chNull };
	DOMImplementation *impl = 
		DOMImplementationRegistry::getDOMImplementation(gLS);
	if (impl==NULL) cout << "No se logro una implementacion" << endl;
    
	//Un DOMBuilder puede parsear XML's y construir el arbol DOM asociado
	//Tambien experimental
	parser = 
		((DOMImplementationLS*)impl)->createDOMBuilder(DOMImplementationLS::MODE_SYNCHRONOUS, 0);
	
	//Puede que necesitemos usar los espacios de nombres	
	parser->setFeature(XMLUni::fgDOMNamespaces, true);
	
	//Se va a usar validacion mediante Schemas
    parser->setFeature(XMLUni::fgXercesSchema, true);
    
	//Si esta caracteristica est a true, el documento
    //debe especificar una gramtica
    parser->setFeature(XMLUni::fgDOMValidation, true);
    
    //Importante!. Sin esta caracterstica activada no se
    //har la normalizacin as que <voltaje> 9 </voltaje>
    //ser recogido como un 9 CON UN ESPACIO DELANTE, lo que
    //hace al elemento invlido, ya que no es un nmero puro
    //sino una cadena que tiene un espacio y un 9. Si se 
    //activa esta caracterstica, el motor normaliza los elementos
    //y los recoge como manda el esquema.
    parser->setFeature (XMLUni::fgDOMDatatypeNormalization, true);

	//Se va a usar la validacin exhaustiva de Schemas
    //Estas comprobaciones extras pueden consumir mucha memoria
    //o recursos pero solo lo hacemos una vez (merece la pena)
    //parser->setFeature(XMLUni::fgXercesSchemaFullChecking, true);
    
    //Se instala un nuevo gestor de errores
    manejadorErrores = new DOMGestionError();
    parser->setErrorHandler ( manejadorErrores );
}
bool Parser::esValido(const char* fichero){
	const XMLCh* nombrefichero;
	
	cout << "Analizando " << fichero << endl;
	nombrefichero=XMLString::transcode (fichero);
		
	//Se analiza un fichero y se recupera el objeto
	//DOMDocument que representa los elementos
	//XML que hay en el fichero
	documento=parser->parseURI (nombrefichero);
        
    // El gestor de errores nos dice si ha habido errores
    // y esta funcion nos dice si el fichero es vlido
    // por lo que el fichero es vlido si no han habido errores
    // De ah la negacin
	return ! (this->haHabidoErrores() );
}

bool Parser::haHabidoErrores()
{
	DOMGestionError* manejador= (DOMGestionError*)
		parser->getErrorHandler();
	
	return (  manejador->hayErrores() );
}

char* Parser::obtenerError()
{
	DOMGestionError* manejador= (DOMGestionError*)
		parser->getErrorHandler();
	return manejador->getError();
}
vector <Elemento*> Parser::extraerElementos ()
{
	vector<Elemento*> vector_elementos;
	/*	Cuando se llama a este mtodo ya se dispone de un objeto DOMDocument
		almacenado en "documento", por lo que podemos acceder a los elementos
		XML almacenados*/
		
		/*Obtenemos el nodo raiz que contiene a todos los demas*/
		DOMNode*	raiz; 
		raiz=(DOMNode*) documento->getDocumentElement();
		
		/*Vamos al primer hijo...*/
		DOMNode*	primer_elemento;
		primer_elemento= raiz->getFirstChild();
		
		/*...y recorremos todos los hijos*/
		DOMNode*	elemento_siguiente=primer_elemento;
		while (elemento_siguiente!=NULL)
		{
			if (elemento_siguiente->getNodeType()==DOMNode::ELEMENT_NODE)
			{
				char* nombre=XMLString::transcode
					(elemento_siguiente->getNodeName());
					
				DOMElement* elemento_A_Pasar=(DOMElement*) elemento_siguiente;
				
				if (XMLString::equals (nombre, "motor"))
				{
					Motor *m=new Motor();
					m->construir ( elemento_A_Pasar );
					vector_elementos.push_back(m);
				}
				if (XMLString::equals (nombre, "biestable"))
				{
					Biestable *b=new Biestable();
					b->construir ( elemento_A_Pasar );
					vector_elementos.push_back(b);
				}
				if (XMLString::equals (nombre,"lampara"))
				{
					Lampara* l=new Lampara();
					l->construir ( elemento_A_Pasar );
					vector_elementos.push_back(l);
				}
				if (XMLString::equals (nombre, "reed"))
				{
					Reed* r=new Reed();
					r->construir ( elemento_A_Pasar );
					vector_elementos.push_back(r);
				}
				if (XMLString::equals (nombre, "fotosensor"))
				{
					FotoSensor *f=new FotoSensor();
					f->construir ( elemento_A_Pasar );
					vector_elementos.push_back (f);
				}
				if (XMLString::equals (nombre, "pulsador"))
				{
					Pulsador* p=new Pulsador();
					p->construir ( elemento_A_Pasar );
					vector_elementos.push_back(p);
				}
				if (XMLString::equals (nombre, "electroiman"))
				{
					Electroiman* e=new Electroiman();
					e->construir ( elemento_A_Pasar );
					vector_elementos.push_back(e);
				}
				if (XMLString::equals (nombre, "zumbador"))
				{
					Zumbador* z=new Zumbador();
					z->construir ( elemento_A_Pasar );
					vector_elementos.push_back(z);
				}
				
				if (XMLString::equals (nombre, "rele"))
				{
					Rele* r=new Rele();
					r->construir ( elemento_A_Pasar );
					vector_elementos.push_back(r);
				}
				/*
				Por terminar
				if (XMLString::equals (nombre, "temporizador"))
				{
					Temporizador* t=new Temporizador();
					t->construir (elemento_siguiente);
					vector_elementos.push_back(t);
				}
				*/
				if (XMLString::equals (nombre, "contador"))
				{
					Contador* c=new Contador();
					c->construir ( elemento_A_Pasar );
					vector_elementos.push_back(c);
				}
				/*
				if (XMLString::equals (nombre, "sistema"))
				{
					Sistema* sistema = new Sistema ();
					sistema->construir ( elemento_A_Pasar );
					vector_elementos.push_back ( sistema );
				}
				*/
			} /* Fin del if que comprueba qu elemento hemos encontrado */
			
			elemento_siguiente=elemento_siguiente->getNextSibling();
		}
		return vector_elementos;
}


vector <Suscripcion*> Parser::extraerRelaciones ()
{
	vector<Suscripcion*> vector_relaciones;
	/*	Cuando se llama a este mtodo ya se dispone de un objeto DOMDocument
		almacenado en "documento", por lo que podemos acceder a los elementos
		XML almacenados*/
		
		/*Obtenemos el nodo raiz que contiene a todos los demas*/
		DOMNode*	raiz; 
		raiz=(DOMNode*) documento->getDocumentElement();
		
		/*Vamos al primer hijo...*/
		DOMNode*	primer_elemento;
		primer_elemento= raiz->getFirstChild();
		
		/*...y recorremos todos los hijos*/
		DOMNode*	elemento_siguiente=primer_elemento;
		while (elemento_siguiente!=NULL)
		{
			if (elemento_siguiente->getNodeType()==DOMNode::ELEMENT_NODE)
			{
				char* nombre=XMLString::transcode
					(elemento_siguiente->getNodeName());
				
				if (XMLString::equals (nombre, "relacion"))
				{
					Suscripcion* s=new Suscripcion();
					s->construir ( (DOMElement*) elemento_siguiente);
					vector_relaciones.push_back (s);
				}
				
			}
			
			elemento_siguiente=elemento_siguiente->getNextSibling();
		}
		return vector_relaciones;
}


Sistema* Parser::getSistema()
{
	/*	Esta es el vector de elementos que nos devuelve el Parser*/
	vector<Elemento*> vectorElementos;
	
	/*	Este es el vector de relaciones que nos devuelve el Parser */
	vector <Suscripcion*> vectorRelaciones;
	
	/*	Este sistema ser el que creemos aqui y devolvamos. Representa
		un objeto Sistema leido de un documento XML previamente analizado */
	Sistema* sistemaADevolver= new Sistema();
	
	
	/*	Se extraen los elementos y las relaciones entre ellos*/
	vectorElementos		=	this->extraerElementos  ();
	//cout << "Elementos extraidos" << endl;
	/* Aun me queda implementar esto*/
	vectorRelaciones	=	this->extraerRelaciones ();
	
	
	
	/*	Vamos insertando los elementos en el sistema */
	
	vector<Elemento*>::iterator elemento;
	
	for ( 	elemento=vectorElementos.begin();
			elemento!=vectorElementos.end();
			elemento++ )
	{
		sistemaADevolver->anadirElemento (*elemento);
		Elemento* e=*elemento;
		printf ("En el sistema se aade el elemento %s\n",
			e->getNombreElemento());
	}
	
	/*	E insertamos tambien todas las relaciones encontradas en el sistema */
	
	vector<Suscripcion*>::iterator relacion;
	for (	relacion = vectorRelaciones.begin();
			relacion!= vectorRelaciones.end();
			relacion++ )
	{
		/*	TODO: aadir las relaciones entre elementos XML */
	}
	return sistemaADevolver;
}

/*	Suponiendo que ya se haya suministrado un documento este
			mtodo se sita al comienzo del documento */
DOMNode* Parser::inicializarRecorrido()
{
	assert (parser!=NULL);
	assert (documento!=NULL);
	
	nodoRaiz=(DOMNode*) documento->getDocumentElement();
	
	return nodoRaiz;
}

void Parser::escribir (Elemento* elemento, char *fichero)
{
	LocalFileFormatTarget* destino=
		new LocalFileFormatTarget (fichero);	
		
	/*	Obtenemos una implementacion de DOM que pueda hacer
		Load and Save */
	static const XMLCh gLS[] = { chLatin_L, chLatin_S, chNull };
	DOMImplementation *impl = 
		DOMImplementationRegistry::getDOMImplementation(gLS);
	DOMWriter* escritor=((DOMImplementationLS*)impl)->createDOMWriter();
	cout << "Obteniendo nodo para la escritura en " << fichero <<
		" del elemento " << elemento->getNombreElemento() << endl;
		

	
	cout << "Creado documento vacio en parser" << endl;
	NodoXML nodoElemento= elemento->getXML ( impl );
	cout << "Nodo obtenido!\n";
	/*	FormatPrettyPrint ordena la escritura usando tabuladores*/
	escritor->setFeature (XMLUni::fgDOMWRTFormatPrettyPrint, 
				   			true);
	escritor->writeNode (destino, *nodoElemento);
		
	delete destino;
	delete escritor;
}
Parser::~Parser()
{
	delete parser; 
	delete manejadorErrores;
	XMLPlatformUtils::Terminate();
}
