#include "sistema.h"
#include "conector.h"
#include "codificador.h"


Sistema::Sistema()
{

	/*	refactorizacion*/
	for (int i=0;i<MAX_LONG_VECTOR_ENTRADAS;i++)
	{
		Conector *c=new Conector();
		c->setNombreElemento ("Entrada");
		conectores_entrada.push_back(c);		
	}
	
	for (int i=0;i<MAX_LONG_VECTOR_SALIDAS;i++)
	{
		Conector *c=new Conector();
		c->setNombreElemento ("Salida");
		conectores_salida.push_back(c);		
	}
	
	/*	El sistema tendr su propio esquema de correspondencias*/
	correspondencia_de_entradas.clear();
	correspondencia_de_salidas.clear();
	/* Se asigna una cantidad de memoria. Por ejemplo 20 posiciones */
	correspondencia_de_entradas.reserve(20);
	correspondencia_de_salidas.reserve(20);
	totalEntradas = 0;
	totalSalidas = 0;
	nombreEntradas.clear();
	nombreSalidas.clear();
}
Sistema::~Sistema()
{
	printf ("\nSistema %s: invocado destructor\n", nombreElemento);
	destruirElementos();
	printf ("Sistema: destructor ejecutado\n");

}
/*	En los sistemas, las suscripciones se hacen a los conectores de
	salida y no al sistema directamente*/
int Sistema::anadirSuscriptor   (Elemento* e,int pos_entrada, 
							int pos_salida)
{
	bool elemento_es_null = (e==NULL);
	assert (!elemento_es_null);
	assert (pos_entrada>=0);
	assert (pos_salida>=0);
	Conector* conectorSalida = conectores_salida.at(pos_salida);
//	const char* nom=e->getNombreElemento();
	//printf ("El elemento %s conecta su entrada %i al conector %i(%x)", nom,
//		pos_entrada, pos_salida, conectorSalida);
	/*	Hack: Se hacen dos suscripciones, una del conector de salida
		hacia el elemento y otra del sistema propiamente dicho hacia
		el elemento */
	conectorSalida->anadirSuscriptor (e, pos_entrada);
	int codigo=Elemento::anadirSuscriptor (e, pos_entrada);
	return codigo;
}
int Sistema::eliminarSuscriptor (Elemento* e,int pos_entrada, 
							int pos_salida)
{
	bool elemento_es_null = (e==NULL);
	assert (!elemento_es_null);
	assert (pos_entrada>=0);
	assert (pos_salida>=0);

	Conector* conectorSalida = conectores_salida.at(pos_salida);
	conectorSalida->eliminarSuscriptor (e, pos_entrada, pos_salida);
	int codigo=Elemento::eliminarSuscriptor (e, pos_entrada, pos_salida);
	return codigo;
}
int Sistema::setEntrada (Senal* s, int posicion)
{
	bool senal_es_nula = (s==NULL);
	assert (!senal_es_nula);
	
	/*	Se busca en la correspondencia entre entradas un
		elemento cuya posicion de entrada en el sistema
		sea la que nos pasan como argumento */
	vector<Correspondencia>::iterator c;
	
	for ( 	c= correspondencia_de_entradas.begin();
			c!=correspondencia_de_entradas.end();
			c++ )
	{
		Correspondencia c_actual=*c;
		int posicion_en_sistema = c_actual.getPosSistema();
		if (posicion_en_sistema == posicion)
		{
			//Elemento* elemento=c_actual.getElemento();
//			printf ("Listo para pasar la senal al elemento\n");
//			elemento->setEntrada(s);
			Conector* conector=c_actual.getConector();
			//printf ("Se mete la senal %s en la posicion %i (conector %x)\n",
//				s->toString(), posicion, conector);
			conector->setEntrada (s);
			return 0;
		}
	} /* Fin del for que recorre las correspondencias de entradas */
	printf ("No habia nada en esa posicion\n");
	return POS_ENTRADA_INVALIDA;
}


void Sistema::Conectar (vector<Conector*> &conectores, Elemento* elemento,
			int totalPosicionesElemento, 
			vector<Correspondencia> &correspondencias, TipoAccion accion, 
			int primeraPosicionAConectar)
{
	if (numConectoresLibres(conectores)<totalPosicionesElemento)
	{
		printf ("\n\nNo se puede aadir el elemento, hay demasiados");
		printf (" y no hay conectores libres en el sistema con");
		printf (" el nombre %s\n\n", nombreElemento);
		exit (-1);
	}
	
	int pos=primeraPosicionAConectar;
	int numConexion=0;
	
	Conector* conector=conectores.at(pos);
	if (!conector->estaLibre())
	{
		/* Si no est libre esa posicin ha ocurrido un error grave */
		assert (false);
	}
	/*	Se van conectando las salidas de los conectores que se encuentren
		libres a las posiciones de entrada del elemento e*/
	while (numConexion<totalPosicionesElemento)
	{
		Conector* conector=conectores.at(pos);
		if (conector->estaLibre())
		{
			enlazar (conector, elemento, numConexion, accion, pos);
			
			/*	La posicion pos del elemento e ocupa
				la posicion i en este sistema */
			Correspondencia c(elemento, numConexion, pos, conector);
//			printf ("El elemento %s\n", elemento->getNombreElemento());
//			printf ("\t Asocia su pos %i a la pos de sistema %i(%x)\n",
//				numConexion, pos, conector );
			correspondencias.push_back(c);
			numConexion++;
		} /* Fin del if */
		/*	Se examina la posicion siguiente */
		pos++;
	}
////	printf ("El valor de pos es %i\n", pos);
}

void Sistema::enlazar 
	(Conector* c, Elemento* e, int pos_elemento, TipoAccion accion, int pos_sistema)
{
	assert (e!=NULL);
	assert (c!=NULL);
	assert (pos_elemento>=0);
	switch (accion)
	{
		case CONECTAR_ENTRADAS_DEL_ELEMENTO:
			{
//				printf ("Conectando entrada %i del elemento\n", pos_elemento);
				c->anadirSuscriptor (e, pos_elemento);
				
				/*	Se aprovecha para almacenar el nombre de 
					la entrada de este elemento en el map llamado
					"nombreentradas"*/
				string nombreelemento(e->getNombreElemento());
				nombreEntradas[nombreelemento]=pos_sistema;
				
				/*	El elemento toma nota de la posicion del sistema
					si no lo habia hecho antes */
				if (!e->entradasEstanMapeadas())
				{
					e->setMapeoPrimeraEntrada(pos_sistema);
				}
				break;
			} /* Fin del case de conexion de entradas */
		case CONECTAR_SALIDAS_DEL_ELEMENTO:
			{
				assert (e!=NULL);
				assert (pos_elemento<=1);
				e->anadirSuscriptor (c, POS_ENTRADA, pos_elemento);
				
				string nombreelemento(e->getNombreElemento());
				nombreSalidas[nombreelemento]=pos_sistema;
				/*	El elemento toma nota de la posicion del sistema
					si no lo habia hecho antes */
				if (!e->salidasEstanMapeadas())
				{
					e->setMapeoPrimeraSalida(pos_sistema);
				}
					
				break;
			} /* Fin del case de conexion de salidas */
	} /* Fin del switch */
	c->marcarComoUsado();
	return ;
}
/*	Segunda implementacin de anadirElemento*/
int Sistema::anadirElemento(Elemento* e)
{
	return this->anadirElemento (e, 0);
}

int Sistema::anadirElemento(Elemento* e,int pos)
{
	assert (e!=NULL);
	
	int totalEntradasElemento	= e->getTotalEntradas();
	assert (totalEntradasElemento>=0);
	
	Conectar (conectores_entrada, e, totalEntradasElemento, 
		correspondencia_de_entradas, 
		CONECTAR_ENTRADAS_DEL_ELEMENTO, pos);
	
	int totalSalidasElemento	= e->getTotalSalidas();
	assert (totalSalidasElemento>=0);
	Conectar (conectores_salida, e, totalSalidasElemento, 
		correspondencia_de_salidas, 
		CONECTAR_SALIDAS_DEL_ELEMENTO, pos);
	/* Se almacena el elemento */
	string nombre(e->getNombreElemento());
	elementos[nombre]=e;
	
	/*	Y se actualizan los datos relativos al propio sistema */
	this->totalEntradas += totalEntradasElemento;
	this->totalSalidas += totalSalidasElemento;
	return elementos.size();
}

/* Segunda implementacin de eliminarElemento*/
int Sistema::eliminarElemento(Elemento* elementoPasado)
{
	assert (elementoPasado!=NULL);
	/* En principio no hay nada eliminado */
	int codigo = NO_SE_ENCONTRO_NADA_PARA_ELIMINAR ;
	
	/*	El elemento notifica a todos los conectores de entrada
		que deben borrar sus referencias a este elemento */
	elementoPasado->notificarEliminacion();		
	
	/*	Se liberan todos los conectores de salida */
	vector<Correspondencia>::iterator c;
	
	for (	c = correspondencia_de_salidas.begin();
			c!= correspondencia_de_salidas.end();
			c++)
	{
		Elemento* elementoActual= c->getElemento();
		assert (elementoActual!=NULL);
		/*	Destruimos el conector*/
		if ( elementoActual==elementoPasado )
		{
			Conector* conector=c->getConector();
			conector->liberar();
			printf ("Se ha encontrado una correspondencia\n");
			
			/*	Se ha encontrado algo que se puede eliminar, aunque
				esto no significa que todo el elemento est eliminado */
			codigo = ELEMENTO_ELIMINADO ;
		}
	} /* Fin del for que libera los conectores de salida */
		
	borrarReferencias (elementoPasado);
	return codigo;
}

int Sistema::destruirElemento (Elemento* e)
{
	bool elemento_es_nulo = (e==NULL);
	assert (!elemento_es_nulo);
	/* TODO: Probar esto bien */
	/* Ya lo he probado mucho pero quiz an se pueda probar ms */
	eliminarElemento(e);
	
	/*	Esta es la unica diferencia con respecto al metodo eliminarElemento*/
	delete e;
	e=NULL;
	
	return 0;
}
int Sistema::setEntrada (Senal* s, const char* nombre, int posicion)
{
	/* TODO: Este mtodo no se usa mucho en la realidad desde
		el entorno grfico de WX. Quiz se pudiera eliminar */
	return 0;
	
}

Senal* Sistema::getEntradaPorNombre (const char* nombre)
{
	Senal* devolver=getEntradaPorNombreYPosicion (nombre, POS_ENTRADA);
	
	return devolver;
}

Senal* Sistema::getSalidaPorNombre (const char* nombre)
{
	Senal* devolver=getSalidaPorNombreYPosicion (nombre, POS_SALIDA);
	
	return devolver;
}

Senal* Sistema::getEntradaPorNombreYPosicion (const char* nombre, int pos)
{
	Senal* entrada;
	/*	Para cada elemento se busca una correspondencia
		con ese nombre y la posicion estandar */
	
	Correspondencia c;
	map<string, Elemento*>::iterator i;
	
	for (	i=elementos.begin(); 
			i!=elementos.end();
			i++ )
	{
		Elemento *e = i->second;
		entrada = e->getEntradaPorNombreYPosicion (nombre, pos);
		if (entrada!=NULL) return entrada;
	} /* Fin del for */
	
	/*	Si no se ha encontrado nada se devuelve esto, que suele
		ser NULL  */
	return entrada;
}


Senal* Sistema::getSalidaPorNombreYPosicion (const char* nombre, int pos)
{
	Senal* salida;
	/*	Para cada elemento se busca una correspondencia
		con ese nombre y la posicion estandar */
	
	Correspondencia c;
	map<string, Elemento*>::iterator i;
	
	for (	i=elementos.begin(); 
			i!=elementos.end();
			i++ )
	{
		Elemento* e= i->second;
		salida = e->getSalidaPorNombreYPosicion(nombre, pos);
		if (salida != NULL) return salida;
	} /* Fin del for */
	
	/*	Si no se ha encontrado nada se devuelve esto, que
		suele ser NULL en muchos casos */
	return salida;
}

/*	Refactorizacion */
Elemento* Sistema::getRefElementoPorNombre (const char* nombreElem)
{
	Elemento* referencia;
	/*	Se recorren los elementos, preguntndoles a todos
		ellos si pueden devolver una referencia con dicho
		nombre */
	referencia=NULL;
	
	map<string, Elemento*>::iterator i;
	
	for (	i=elementos.begin(); 
			i!=elementos.end();
			i++ )
	{
		Elemento* e=i->second;
		
		referencia = e->getRefElementoPorNombre(nombreElem);
		
		/*	Si se encuentra una referencia se termina inmediatamente */
		if (referencia!=NULL) return referencia;
	} /* Fin del for */
	
	/*	Si se llega aqu, probablemente se devuelve NULL */
	return referencia ;
}
int Sistema::getPrimeraEntrada(const char* nombreElem)
{
	int posicion;
	map<string, Elemento*>::iterator i;
	
	for (	i=elementos.begin(); 
			i!=elementos.end();
			i++ )
	{
		Elemento* e=i->second;
		
		posicion = e->getPrimeraEntrada(nombreElem);
		
		if (posicion != ELEMENTO_NO_ENCONTRADO)
		{
			Correspondencia c= buscarPosElemento (correspondencia_de_entradas,
				e, posicion);
			int posicionDeSistema = c.getPosSistema();
			return posicionDeSistema;
		}
	} /* Fin del for */
	/* Si no se encuentra nada... */
	return ELEMENTO_NO_ENCONTRADO;
}

int Sistema::getPrimeraSalida (const char* nombreElem)
{
	int posicion;
	map<string, Elemento*>::iterator i;
	
	for (	i=elementos.begin(); 
			i!=elementos.end();
			i++ )
	{
		Elemento* e=i->second;
		
		posicion = e->getPrimeraSalida(nombreElem);
		
		if (posicion != ELEMENTO_NO_ENCONTRADO)
		{
			Correspondencia c= buscarPosElemento (correspondencia_de_salidas,
				e, posicion);
			int posicionDeSistema = c.getPosSistema();
			return posicionDeSistema;
		}
	} /* Fin del for */
	
	/* Si no se encuentra nada... */
	return ELEMENTO_NO_ENCONTRADO;
}
int Sistema::getPrimeraEntradaUtil (const char* nombreElem)
{
	int posicion;
	map<string, Elemento*>::iterator i;
	
	for (	i=elementos.begin(); 
			i!=elementos.end();
			i++ )
	{
		Elemento* e=i->second;
		
		posicion = e->getPrimeraEntradaUtil(nombreElem);
		/*	Cuando se encuentra un elemento se busca la entrada de sistema
			asociada a ese elemento y esa entrada del elemento (o salida) */
		if (posicion != ELEMENTO_NO_ENCONTRADO)
		{
			Correspondencia c= buscarPosElemento (correspondencia_de_entradas,
				e, posicion);
			int posicionDeSistema = c.getPosSistema();
			return posicionDeSistema;
		}
	} /* Fin del for */
	
	/* Si no se encuentra nada... */
	return ELEMENTO_NO_ENCONTRADO;
}
int Sistema::getPrimeraSalidaUtil (const char* nombreElem)
{
	int posicion;
	map<string, Elemento*>::iterator i;
	
	for (	i=elementos.begin(); 
			i!=elementos.end();
			i++ )
	{
		Elemento* e=i->second;
		
		posicion = e->getPrimeraSalidaUtil(nombreElem);
		
		if (posicion != ELEMENTO_NO_ENCONTRADO)
		{
			Correspondencia c= buscarPosElemento (correspondencia_de_salidas,
				e, posicion);
			int posicionDeSistema = c.getPosSistema();
			return posicionDeSistema;
		}
	} /* Fin del for */
	
	/* Si no se encuentra nada... */
	return ELEMENTO_NO_ENCONTRADO;
}
/*	Estos metodos son heredados de Elemento y son redefinidos
	para que devuelvan la salida del conector correspondiente*/


int Sistema::getEntradaDeSistema (const char* nombrePasado, 
			int posicionPasada)
{
	Correspondencia c;
	map<string, Elemento*>::iterator i;
	int posicion;
	for (	i=elementos.begin(); 
			i!=elementos.end();
			i++ )
	{
		Elemento* e=i->second;
		posicion = e->getEntradaDeSistema (nombrePasado, posicionPasada);
		if (posicion!=-1)
		{
			Correspondencia c= buscarCorrespondencia 
				(correspondencia_de_entradas, e->getNombreElemento(), posicionPasada);
			int posicionDeSistema = c.getPosSistema();
			return posicionDeSistema;
		}
	} /* Fin del for */
	
	/*	Si no se ha encontrado nada se devuelve esto */
	return -1;
		
}

int Sistema::getSalidaDeSistema (const char* nombrePasado, 
			int posicionPasada)
{
	Correspondencia c;
	map<string, Elemento*>::iterator i;
	int posicion;
	for (	i=elementos.begin(); 
			i!=elementos.end();
			i++ )
	{
		Elemento* e=i->second;
		posicion = e->getSalidaDeSistema (nombrePasado, posicionPasada);
		if (posicion!=-1)
		{
			Correspondencia c= buscarCorrespondencia 
				(correspondencia_de_salidas, e->getNombreElemento(), posicionPasada);
			int posicionDeSistema = c.getPosSistema();
			return posicionDeSistema;
		}
	} /* Fin del for */
	
	/*	Si no se ha encontrado nada se devuelve esto */
	return -1;
}	

Correspondencia Sistema::buscarCorrespondencia (
			vector<Correspondencia> &v, const char* nombre,
			int posicionElemento )
{
	vector<Correspondencia>::iterator i;
	/*	Esto es lo que se devuelve si no se encuentra */
	Correspondencia cInvalida(NULL, -1, -1, NULL);
	
	for (	i=v.begin(); 
			i!=v.end();
			i++ )
	{
		Correspondencia c=*i;
		Elemento* elemEstudiado=c.getElemento();
		
		string nombreDelElementoEstudiado=string
			(elemEstudiado->getNombreElemento());
		string nombrePasado(nombre);
		//cout << "Se compara nombrePasado " << nombrePasado;
//		cout << " con el nombre " << nombreDelElementoEstudiado << endl;
		if (nombrePasado==nombreDelElementoEstudiado)
		{
			if (posicionElemento==c.getPosElemento())
			{
				return c;
			}
		}
	}
	return cInvalida;
}

Senal* Sistema::getEntradaPorPosicion(int posicion)
{
	Correspondencia c=buscarPosSistema (correspondencia_de_entradas,posicion);
	
	Conector* conector= c.getConector();
	
	Senal* senalEntrada = conector->getEntradaPorPosicion();
	return senalEntrada;
	
}

Senal* Sistema::getSalidaPorPosicion (int posicion)
{
	Correspondencia c=buscarPosSistema (correspondencia_de_salidas,posicion);
	
	Conector* conector= c.getConector();
	
	Senal* senalSalida = conector->getSalidaPorPosicion();
	return senalSalida;
	
}

/*	Cuidado: se asume que un elemento puede tener muchas
	entradas o salidas pero solo de un tipo. Si no fuera as
	habra que modificar el cdigo para buscar el conector
	conectado a la entrada de ese elemento y averiguar el tipo
	de seal que maneja ese conector */
int Sistema::getTipoEntrada(int posicion)
{
	Correspondencia c= buscarPosSistema (correspondencia_de_entradas, posicion);
	Elemento* e= c.getElemento();
	assert (e!=NULL);
	int tipo = e->getTipoEntrada();
	return tipo;
}

int Sistema::getTipoSalida (int posicion)
{
	Correspondencia c= buscarPosSistema (correspondencia_de_salidas, posicion);
	Elemento* e= c.getElemento();
	int tipo = e->getTipoSalida();
	return tipo;
}

bool Sistema::contieneElemento (const char* nombre, 
	Elemento* elementoADevolver)
{
	string nombrecpp(nombre);
	map<string, Elemento*>::iterator i=elementos.find(nombre);
	
	if (i==elementos.end())
	{
		elementoADevolver=NULL;
		return false;
	}
	else {
		elementoADevolver=i->second;
		return true;
	}
}

map <string, Elemento*> Sistema::getElementos()
{
	return elementos;
}

int	Sistema::numElementos()
{
	return elementos.size();
}
int Sistema::numConectoresLibres (vector<Conector*> v)
{
	/*	TODO: Implementar este mtodo */
	return 100;
}
int Sistema::buscarConector (Conector* vectorConectores[], Elemento* e, 
				int posicion, int limMaxVector)
{
	int posicionADevolver=ELEMENTO_NO_ENCONTRADO;
	
	for (	int numConector=0;
			numConector < limMaxVector;
			numConector++)
	{
		Conector* conectorExaminado=vectorConectores[numConector];
		/*	Algunos de los conectores podrian no ir conectados a nada
			y por tanto ser NULL, en ese caso los saltamos*/
		if (conectorExaminado!=NULL)
		{
			/*	Ahora buscamos la suscripcion que involucra a ese conector
			como avisador*/
			Elemento* elem_Examinado= 
				conectorExaminado->getElementoSuscriptor();
			int posEntradaDelExaminado=
				conectorExaminado->getPosEntradaSuscriptor();
			
			if ( (posEntradaDelExaminado==posicion) && (elem_Examinado==e) )
			{
				posicionADevolver=numConector;
				return posicionADevolver;
			}
		} /*Fin del if que comprueba si conectorExaminadoEsNULL*/
	} /*Fin del for*/
	
	return posicionADevolver; /*	Cuidado, podria ser ELEMENTO_NO_ENCONTRADO*/
}

Correspondencia Sistema::buscarPosSistema (vector<Correspondencia> &c, int pos)
{
	assert (pos>=0);
	vector<Correspondencia>::iterator i;
	Correspondencia cVacia (NULL, -1, -1, NULL);
	for ( 	i=c.begin();
			i!=c.end();
			i++ )
	{
		Correspondencia cActual = *i;
		if (cActual.getPosSistema() == pos ) {
			return cActual;
		}
	} /* Fin del for */
	return cVacia;
}
Correspondencia Sistema::buscarPosElemento (vector<Correspondencia> &c, 
			Elemento* e, int pos)
{
	assert (pos>=0);
	vector<Correspondencia>::iterator i;
	Correspondencia cVacia (NULL, -1, -1, NULL);
	for ( 	i=c.begin();
			i!=c.end();
			i++ )
	{
		Correspondencia cActual = *i;
		if ( (cActual.getPosElemento() == pos ) && (cActual.getElemento()==e) ){
			return cActual;
		}
	} /* Fin del for */
	
	return cVacia;
}

void Sistema::borrarReferencias (Elemento* e)
{
	assert (e!=NULL);
	borrarCorrespondencias (e, correspondencia_de_entradas, e->getTotalEntradas());
	borrarCorrespondencias (e, correspondencia_de_salidas, e->getTotalSalidas());
	borrarNombre (string (e->getNombreElemento()));
}
void Sistema::borrarCorrespondencias (Elemento* e, 
				vector<Correspondencia> &c, int num_borrados)
{
	assert (e!=NULL);
	assert (num_borrados>=0);
	
	vector<Correspondencia>::iterator i;
	int correspondencias_borradas=0;
	i=c.begin();
	
	/*	Esto es un correspondencias_borradas<=num_borrados) )??*/
	while (i!=c.end() && (correspondencias_borradas<num_borrados) )
	{
		Correspondencia cActual = *i;
		if (cActual.getElemento()==e) {
			printf ("Borrando una correspondencia\n");
			/*	La operacion erase MODIFICA el vector por lo que de esta
				forma nos aseguramos que el borrado realmente se hace 
				de principio a fin, ya que erase devuelve un iterador */
			i=c.erase(i);
			correspondencias_borradas++;
		}
		i++;
	} /* Fin del for */
	
}

void Sistema::borrarNombre (string nombre)
{
	assert (nombre!="");
	map <string, Elemento*>::iterator pos_nombre;
	pos_nombre = elementos.find(nombre);
	if (pos_nombre!=elementos.end()) elementos.erase (pos_nombre);
	/*TODO: Es solo una prueba. Ocurre algo?*/
	/*SI: Esto da un error:elementos.erase (elementos.end());*/
}

void Sistema::destruirElementos()
{
	/* 	No es un destructor demasiado perfecto, pero libera la mayor parte
		de memoria asignada, ya que libera todos los elementos contenidos. */
	map<string, Elemento*>::iterator i;
	for (	i=elementos.begin();
			i!=elementos.end();
			i++)
	{
		Elemento* elem_a_destruir = i->second;
		assert (elem_a_destruir!=NULL);
		delete elem_a_destruir;
	} /* Fin del for */
}
