#ifndef _sistema_
#define _sistema_

#include "constantes.h"
#include "elemento.h"
#include "conector.h"
#include <map>
#include <string>


using namespace std;

class Sistema : public Elemento
{
	public:
		Sistema();
		~Sistema();
		/*	En sistemas es necesario modificar la forma de aadir
			suscripciones. Consultar los detalles de implementacion
			en sistema.cpp*/
			
		int anadirSuscriptor   (Elemento* e,int pos_entrada=POS_ENTRADA, 
							int pos_salida=POS_SALIDA);
		int eliminarSuscriptor (Elemento* e,int pos_entrada=POS_ENTRADA, 
							int pos_salida=POS_SALIDA);
							
		/*	Introduce una seal en una cierta posicion del
			vector de entradas. El llamador asume la responsabilidad
			de que sea la correcta*/
		int setEntrada (Senal* s, int posicion);
		
		/* 	Este mtodo permite modificar la seal de entrada basndonos
			en un nombre y poner esa seal en la posicion de entrada de
			ese elemento*/
		int setEntrada (Senal* s, const char* nombre, int posicion=POS_ENTRADA);
		
		/*	Incorpora un elemento al sistema */
		int anadirElemento(Elemento* e);
		
		/* 	Anade un elemento al sistema pero empezando a conectarlo
			a partir de la posicion pos*/
		int anadirElemento(Elemento* e, int pos);
		
		/*	Elimina un elemento del sistema */
		int eliminarElemento (Elemento* e);
		
		/*	Elimina toda referencia a este elemento */
		int destruirElemento (Elemento* e);
		/*	Nos devuelve la entrada de un elemento dado su nombre*/
		Senal* getEntradaPorNombre (const char* nombre);
		
		/*	Nos devuelve la salida de un elemento dado su nombre*/
		Senal* getSalidaPorNombre (const char* nombre);
		
		/*	Nos devuelve la entrada de un elemento dado su nombre*/
		Senal* getEntradaPorNombreYPosicion (const char* nombre, int pos);
		
		/*	Nos devuelve la salida de un elemento dado su nombre*/
		Senal* getSalidaPorNombreYPosicion (const char* nombre, int pos);
		
		Elemento* getRefElementoPorNombre (const char* nombreElemento);
		
		/*	Nos devuelve la posicion que en el vector de entradas o salidas
			de un cierto elemento. Si es un elemento multientrada o multisalida
			las siguientes posiciones son las siguientes entradas*/
		int getPrimeraEntrada(const char* nombreElemento);
		int getPrimeraSalida (const char* nombreElemento);
		
		/* 	Estos metodos se redefinen aqui porque proporcionan mucha comodidad
			al no tener que sumar todo el rato POS_ENTRADA. Todos los
			elementos tienen una entrada ENABLE (posicion 0), una entrada
			RESET (posicion 1) y una entrada KEEP (posicion 2). La primera
			entrada til es la 3 aunque podra no ocurrir siempre as, razn
			por la que se ofrece este mtodo.*/
		int getPrimeraEntradaUtil (const char* nombreElemento);
		int getPrimeraSalidaUtil  (const char* nombreElemento);
		
		/*	Estos dos mtodos son una de las piedras angulares
			de esta clase. Dado un nombre de elemento y una
			entrada o salida, nos dicen cual es la entrada o
			salida del sistema que corresponden a dicha
			posicion */
		int getEntradaDeSistema (const char* nombreElemento, 
			int posicion=POS_ENTRADA);
		int getSalidaDeSistema (const char* nombreElemento, 
			int posicion=POS_SALIDA);
		
		/*	Este mtodo busca la correspondencia relativa a la
			posicion de entrada o de salida de un elemento.
			La mayor parte de las veces devuelve la misma
			posicion, ya que las correspondencias son muy
			simples en los elementos. Esto se usa para que
			lo puedan heredar todos los elementos y la clase
			Sistema */
		virtual Correspondencia buscarCorrespondencia (
			vector<Correspondencia> &v, const char* nombre,
			int posicionElemento );
			
		
		/*	Estos metodos van a ser ligeramente redefinidos*/
		Senal* getEntradaPorPosicion(int posicion=POS_ENTRADA);
		Senal* getSalidaPorPosicion (int posicion=POS_SALIDA);
		
		virtual int 	getTipoEntrada(int posicion=POS_ENTRADA);
		virtual int 	getTipoSalida (int posicion=POS_SALIDA);
	
		/*	Solo hace falta redefinir uno. El otro nunca ha hecho falta
		virtual bool contieneElemento (Elemento* e);*/
		virtual bool contieneElemento (const char* nombre, Elemento* e);
			
		virtual NodoXML getXML ( DOMImplementation* impl );
		/*	Este metodo devuelve un char* con el XML que representa
			este sistema */
		char*	getCadenaXML ();
		
		/*	Devuelve los elementos conocidos por este sistema*/
		map<string, Elemento*> getElementos();
		int	numElementos();
	protected:
		/* 	Muy dependiente de C++. Algunos compiladores podran no
			soportar bien el uso de maps. En ese caso hay que modificar
			la forma de encontrar la posicion de un elemento en funcion
			de su nombre */

		
		/*	Refactorizacin. */
		map<string, Elemento*> elementos;
		
		
		
		/*	Conectores de que dispone este sistema*/
		vector<Conector*> conectores_entrada;
		vector<Conector*> conectores_salida;
	private:
		int numConectoresLibres (vector<Conector*> v);
		
		/*	Esta funcion busca un conector en un vector de conectores
			cuya suscripcion se haya hecho con el elemento e, y avisando 
			a esa posicion*/
		int buscarConector (Conector* vector[], Elemento* e, 
				int posicion, int limMaxVector);
				
		/*	Conectar un elemento a los conectores
			de entrada o a los de salida en funcion
			del parametro conectarEntradas.
			Los vectores se pasan por referencia para 
			mejorar el rendimiento */
		void Conectar (vector<Conector*> &conectores, Elemento* e,
			int totalPosiciones, 
			vector<Correspondencia>& correspondencias, 
			TipoAccion a, int primeraPosicionAConectar);
			
		/* 	Enlaza un conector concreto con el elemento e siendo pos_elemento
			la posicion del elemento y pos_sistema la posicion que ocupa
			en el sistema.
			P.ej si TipoAccion es CONECTAR la entrada se conecta un conector
			de entrada que ocupa la posicion possistema a la entrada del elemento
			e ubicada en la posicion pos_elemento
			
			Puede parecer rebuscado pero ayuda a reducir la cantidad de cdigo */
		void enlazar (Conector* c, Elemento* e, 
			int pos_elemento, TipoAccion a, int pos_sistema);
		
		/*	Busca en las correspondencias de entrada o de salida aquella
			correspondencia que contenga informacion sobre una posicion
			de entrada o de salida del sistema */
		Correspondencia buscarPosSistema (vector<Correspondencia> &c, int pos);
		/*Quiza haga falta implementar esto. Busca la correspondencia que nos
			d informacin sobre la posicion pos del elemento pasado */
		Correspondencia buscarPosElemento (vector<Correspondencia> &c, 
			Elemento* e, int pos);
		
		/*	Este mtodo obliga al sistema a borrar todas las referencias
			que pudiera tener sobre un elemento. Desde el nombre del
			elemento hasta las correspondencias */
		void borrarReferencias (Elemento* e);		
		/*	Este elemento borra del vector todas las correspondencias
			que contengan el elemento e*/
		void borrarCorrespondencias (Elemento* e, vector<Correspondencia> &c,
										int numborrados);
		
		/*	Borra toda referencia a un nombre de elemento*/
		void borrarNombre (string nombreElemento);
		
		/*	Invoca al destructor de todos y cada uno de los elementos*/
		void destruirElementos();
};
#endif
