#include "unit++.h"
#include "reed.h"
#include "electroiman.h"

using namespace unitpp;

namespace
{
	class PruebaReed: public suite
	{
		void PruebaValoresReed()
		{
			Magnetismo *m1=new Magnetismo(false);
			Magnetismo *m2=new Magnetismo(true);
			Voltaje*	v1=new Voltaje (0.0);
			Voltaje*	v2=new Voltaje (5.0);			
			
			Reed* r1=new Reed(m1,v1,m2,v2);
			
			Magnetismo *mLeido=r1->getMagnetismoReposo();
			
			assert_eq ("Magnetismo en reposo del reed",
				m1->getValor(), mLeido->getValor() );
			
			mLeido=r1->getMagnetismoActivacion();
			assert_eq ("Magnetismo en activacion del reed",
				m2->getValor(), mLeido->getValor() );
				
		}
		
		void PruebaLecturaMetodosVirtuales()
		{
			Magnetismo *m1=new Magnetismo(false);
			Magnetismo *m2=new Magnetismo(true);
			Voltaje*	v1=new Voltaje (0.0);
			Voltaje*	v2=new Voltaje (5.0);
			
			Reed* r1=new Reed(m1,v1,m2,v2);
			
			Magnetismo 	*mLeido=(Magnetismo*)r1->getEntradaPorPosicion();
			
			/*	En principio los valores leidos deben ser los de reposo*/
			assert_eq ("Magnetismo en reposo (metodo virtual)",
				m1->getValor(), mLeido->getValor() );
			
		}
		void PruebaConexionAElectroiman()
		{
			/*Creamos el reed*/
			Magnetismo *m1=new Magnetismo(false);
			Magnetismo *m2=new Magnetismo(true);
			Voltaje*	v1=new Voltaje (0.0);
			Voltaje*	v2=new Voltaje (5.0);
			
			Reed* r1=new Reed(m1,v1,m2,v2);
			
			/*Creamos el electroiman*/
			
			Voltaje* vReposo		=	new Voltaje (0.0);
			Voltaje* vActivacion	=	new Voltaje(5.0);
			Magnetismo* mReposo		=	new Magnetismo (false);
			Magnetismo* mActivacion	=	new Magnetismo (true);
			
			Electroiman* e			=	new Electroiman(vReposo, vActivacion,
				mReposo, mActivacion);
				
			/*	El reed est a la escucha del electroiman*/
			e->anadirSuscriptor(r1);
			
			/*	Y al modificar la entrada del electroiman...*/
			e->setEntrada(vActivacion);
			
			/*	...la salida del reed deberia haber cambiado y estar activa*/
			Voltaje *vReed=(Voltaje*) r1->getSalidaPorPosicion();
			
			assert_eq("Conexion electroiman a reed",
				v2->getValor(), vReed->getValor() );
			
			/*	Dejamos el electroiman como estaba...*/
			e->setEntrada(vReposo);
			
			/*	...y el reed deberia desactivarse*/
			vReed=(Voltaje*) r1->getSalidaPorPosicion();
			assert_eq("Conexion electroiman a reed",
				v1->getValor(), vReed->getValor() );
			
			delete m1;
			delete m2;
			delete v1;
			delete v2;
			delete e;
		}
		public:
		PruebaReed(): suite ("Prueba de los reed")
		{
			add ("Prueba Valores del reed", testcase 
			(this, "PruebaPulsadorBiestableLampara", 
			&PruebaReed::PruebaValoresReed));
			
			add ("Prueba metodos virtuales", testcase 
			(this, "PruebaLecturaMetodosVirtuales", 
			&PruebaReed::PruebaLecturaMetodosVirtuales));
			
			add ("Prueba de conexion con electroiman", testcase 
			(this, "PruebaConexionAElectroiman", 
			&PruebaReed::PruebaConexionAElectroiman));
			
			suite::main().add("PruebaConexiones", this);	
		}
	};
	
	PruebaReed *pReed=new PruebaReed();	
}
